==> libbpp-core_2.0.3.orig.tar.gz: bpp-core-2.0.3/src/Bpp/Numeric/Random/Uniform01K.cpp <==
//
// File Uniform01K.cpp
// Author : Sylvain Gaillard
// Last modification : Friday September 24 2004
//

/*
Copyright or © or Copr. CNRS, (November 17, 2004)

This software is a computer program whose purpose is to provide classes
for numerical calculus.

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#include "Uniform01K.h"

#include "../NumTools.h"

using namespace bpp;

const long Uniform01K::MAXNUMBER = 1000000000;
const long Uniform01K::ZERO = 0;
const long Uniform01K::MODSEED = 256434901;

//** Class constructor: *******************************************************/

Uniform01K::Uniform01K(long seed) :
  _it1(), _it2()
{
	setSeed(seed);
}

//** Class destructor: *******************************************************/
Uniform01K::~Uniform01K() {}

//** Other methodes: *********************************************************/

void Uniform01K::setSeed(long seed)
{
	long tmp1, tmp2;
	unsigned short i, ii, j;
	tmp1 = NumTools::abs(MODSEED - NumTools::abs(seed));
	tmp1 %= MAXNUMBER;
	_tab[55] = tmp1;
	tmp2 = 1;

	for (i=1 ; i<=54 ; i++)
  {
		ii = (21 * i) % 55;
		_tab[ii] = tmp2;
		tmp2 = tmp1 - tmp2;

		if (tmp2 < ZERO) tmp2 += MAXNUMBER;

		tmp1 = _tab[ii];
	}

	for (j=1 ; j<=4 ; j++)
		for (i=1 ; i<=55 ; i++)
    {
			_tab[i] -= _tab[1 + (i + 30) % 55];
			if (_tab[i] < ZERO) _tab[i] += MAXNUMBER;
		}
	_it1 = 0;
	_it2 = 31;
}

double Uniform01K::drawNumber() const
{
	if (++_it1 == 56) _it1 = 1;
	if (++_it2 == 56) _it2 = 1;

	long tmp = _tab[_it1] - _tab[_it2];
	if(tmp < ZERO) tmp += MAXNUMBER;
	_tab[_it2] = tmp;
	double r = 1. * tmp / MAXNUMBER;
	//if (r < 0.) return drawNumber(); // In case of negative number, take the next one in the serie.
	//if (r > 1.) return drawNumber(); // In case of number > 1, take the next one in the serie.
	return r;
}

==> libbpp-core_2.4.1.orig.tar.gz: bpp-core-2.4.1/src/Bpp/Numeric/Random/Uniform01K.cpp <==
//
// File Uniform01K.cpp
// Author : Sylvain Gaillard
// Last modification : Friday September 24 2004
//

/*
Copyright or © or Copr. CNRS, (November 17, 2004)

This software is a computer program whose purpose is to provide classes
for numerical calculus.

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#include "Uniform01K.h"

#include "../NumTools.h"

using namespace bpp;

const long Uniform01K::MAXNUMBER = 1000000000;
const long Uniform01K::ZERO = 0;
const long Uniform01K::MODSEED = 256434901;

//** Class constructor: *******************************************************/

Uniform01K::Uniform01K(long seed) :
  _it1(), _it2()
{
	setSeed(seed);
}

//** Class destructor: *******************************************************/
Uniform01K::~Uniform01K() {}

//** Other methodes: *********************************************************/

void Uniform01K::setSeed(long seed)
{
	long tmp1, tmp2;
	unsigned short i, ii, j;
	tmp1 = NumTools::abs(MODSEED - NumTools::abs(seed));
	tmp1 %= MAXNUMBER;
	_tab[55] = tmp1;
	tmp2 = 1;

	for (i = 1 ; i <= 54 ; i++)
  {
		ii = static_cast<unsigned short>((21 * i) % 55);
		_tab[ii] = tmp2;
		tmp2 = tmp1 - tmp2;

		if (tmp2 < ZERO) tmp2 += MAXNUMBER;

		tmp1 = _tab[ii];
	}

	for (j=1 ; j<=4 ; j++)
		for (i=1 ; i<=55 ; i++)
    {
			_tab[i] -= _tab[1 + (i + 30) % 55];
			if (_tab[i] < ZERO) _tab[i] += MAXNUMBER;
		}
	_it1 = 0;
	_it2 = 31;
}

double Uniform01K::drawNumber() const
{
	if (++_it1 == 56) _it1 = 1;
	if (++_it2 == 56) _it2 = 1;

	long tmp = _tab[_it1] - _tab[_it2];
	if (tmp < ZERO) tmp += MAXNUMBER;
	_tab[_it2] = tmp;
	double r = static_cast<double>(static_cast<long double>(tmp) / MAXNUMBER);
	//if (r < 0.) return drawNumber(); // In case of negative number, take the next one in the serie.
	//if (r > 1.) return drawNumber(); // In case of number > 1, take the next one in the serie.
	return r;
}
